@@ -235,7 +235,8 @@ int proxy_tester_clean_up(struct proxy_tester *tester) {
             if (i + 1 < channel_count) {
                 wrapper.bootstrap->on_shutdown(tester->client_connection, 0, wrapper.bootstrap->user_data);
             }
-            aws_mem_release(tester->alloc, wrapper.bootstrap);
+
+            aws_http_client_bootstrap_destroy(wrapper.bootstrap);
         }
     }
 
@@ -307,7 +308,8 @@ int proxy_tester_create_testing_channel_connection(
 
     /* Use small window so that we can observe it opening in tests.
      * Channel may wait until the window is small before issuing the increment command. */
-    struct aws_http1_connection_options http1_options = AWS_HTTP1_CONNECTION_OPTIONS_INIT;
+    struct aws_http1_connection_options http1_options;
+    AWS_ZERO_STRUCT(http1_options);
     struct aws_http_connection *connection =
         aws_http_connection_new_http1_1_client(tester->alloc, true, 256, &http1_options);
     ASSERT_NOT_NULL(connection);
@@ -418,9 +420,9 @@ int proxy_tester_verify_connect_request(struct proxy_tester *tester) {
     snprintf(
         connect_request_buffer,
         AWS_ARRAY_SIZE(connect_request_buffer),
-        "CONNECT " PRInSTR ":%d HTTP/1.1",
+        "CONNECT " PRInSTR ":%u HTTP/1.1",
         AWS_BYTE_CURSOR_PRI(tester->host),
-        (int)tester->port);
+        tester->port);
 
     struct aws_byte_cursor expected_connect_message_first_line_cursor =
         aws_byte_cursor_from_c_str(connect_request_buffer);
@@ -472,7 +474,25 @@ int proxy_tester_send_connect_response(struct proxy_tester *tester) {
 int proxy_tester_verify_connection_attempt_was_to_proxy(
     struct proxy_tester *tester,
     struct aws_byte_cursor expected_host,
-    uint16_t expected_port) {
+    uint32_t expected_port) {
+    ASSERT_BIN_ARRAYS_EQUALS(
+        tester->connection_host_name.buffer,
+        tester->connection_host_name.len,
+        expected_host.ptr,
+        expected_host.len,
+        "Connection host should have been \"" PRInSTR "\", but was \"" PRInSTR "\".",
+        AWS_BYTE_CURSOR_PRI(expected_host),
+        AWS_BYTE_BUF_PRI(tester->connection_host_name));
+
+    ASSERT_TRUE(tester->connection_port == expected_port);
+
+    return AWS_OP_SUCCESS;
+}
+
+int proxy_tester_verify_connection_attempt_was_to_target(
+    struct proxy_tester *tester,
+    struct aws_byte_cursor expected_host,
+    uint32_t expected_port) {
     ASSERT_BIN_ARRAYS_EQUALS(
         tester->connection_host_name.buffer,
         tester->connection_host_name.len,
