@@ -1,5 +1,8 @@
 // Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
 
+mod dynamic_record_sizing;
+mod group_negotiation;
 #[cfg(feature = "pq")]
 mod pq;
+mod record_padding;
