@@ -230,6 +230,7 @@ typedef enum {
     S2N_ERR_ASYNC_MORE_THAN_ONE,
     S2N_ERR_PQ_CRYPTO,
     S2N_ERR_INVALID_CERT_STATE,
+    S2N_ERR_INVALID_SUPPORTED_GROUP_STATE,
     S2N_ERR_INVALID_EARLY_DATA_STATE,
     S2N_ERR_PKEY_CTX_INIT,
     S2N_ERR_FORK_DETECTION_INIT,
@@ -328,6 +329,7 @@ typedef enum {
     S2N_ERR_KTLS_ENABLE,
     S2N_ERR_KTLS_BAD_CMSG,
     S2N_ERR_KTLS_RENEG,
+    S2N_ERR_KTLS_SOCKOPT,
     S2N_ERR_ATOMIC,
     S2N_ERR_KTLS_KEY_LIMIT,
     S2N_ERR_SECURITY_POLICY_INCOMPATIBLE_CERT,
