@@ -86,8 +86,7 @@ function uvinteg {(
     set -eu
     TESTS="${1:-all}"
     apache2_start
-    # TODO: Dynamic Record Sizes needs a rewrite; skip for now.
-    PYTEST_ARGS="--provider-version $S2N_LIBCRYPTO -x -n auto --reruns=2 --durations=10 -rpfs --cache-clear --ignore-glob=*test_dynamic_record_sizes*"
+    PYTEST_ARGS="--provider-version $S2N_LIBCRYPTO -x -n auto --durations=10 -rpfs"
     cd ./tests/integrationv2
     echo -n "Comparing the current list of integ tests against what is checked-in..."
     PYTHONPATH="" uv run pytest --collect-only $PYTEST_ARGS | grep Module > /tmp/uvinteg_tests.txt
@@ -95,10 +94,13 @@ function uvinteg {(
     echo "tests to run match uvinteg_tests.txt."
     if [[ "$TESTS" == "all" ]]; then
         echo "Running all integ tests with uv"
-        PYTHONPATH="" uv run pytest $PYTEST_ARGS --junitxml=../../build/junit/uv_integ.xml
+        # If the first attempt has any failures, try rerunning with only the failed tests.
+        PYTHONPATH="" uv run pytest --cache-clear $PYTEST_ARGS --junitxml=../../build/junit/uv_integ.xml || \
+        PYTHONPATH="" uv run pytest --last-failed --last-failed-no-failures none $PYTEST_ARGS --junitxml=../../build/junit/uv_integ_retry.xml
     else
+        # We're clearing state between test runs when individual tests are specified.
         for test in "$@"; do
-            PYTHONPATH="" uv run pytest $PYTEST_ARGS --junitxml=../../build/junit/$test.xml -k $test
+            PYTHONPATH="" uv run pytest --cache-clear $PYTEST_ARGS --junitxml=../../build/junit/$test.xml -k $test
         done
     fi
 )}
@@ -220,3 +222,39 @@ function apache2_start(){
       echo "Apache is already running. If \"$APACHE2_INSTALL_DIR\" is stale, it might be in an unknown state."
     fi
 }
+
+function rust_configure {(set -e
+    echo "rust_configure: Cleaning previous build"
+    rm -rf build/
+    echo "rust_configure: Configuring with CMake (RelWithDebInfo, intern libcrypto)"
+    cmake -B build . \
+        -D CMAKE_C_COMPILER=clang \
+        -D CMAKE_BUILD_TYPE=RelWithDebInfo \
+        -D BUILD_TESTING=OFF \
+        -D S2N_INTERN_LIBCRYPTO=ON
+)}
+
+function rust_build {(set -e
+    echo "rust_build: Building s2n-tls"
+    cmake --build ./build -j $(nproc)
+    echo "rust_build: Generating Rust bindings (bindgen)"
+    bindings/rust/extended/generate.sh --skip-tests
+)}
+
+function rust_test {(set -e
+    # Set up local Rust toolchain to avoid conflicts between CI and Nix Rust installations.
+    # This ensures we use a consistent, isolated toolchain regardless of environment.
+    echo "rust_test: Setting up local Rust toolchain (rustup stable)"
+    export RUSTUP_HOME=$(pwd)/.rustup
+    export CARGO_HOME=$(pwd)/.cargo
+    export PATH=$(pwd)/.cargo/bin:$PATH
+    rustup set profile minimal
+    rustup set auto-self-update disable
+    rustup toolchain install stable
+    rustup default stable
+    echo "rust_test: Exporting s2n-tls headers and libs for Cargo"
+    export S2N_TLS_LIB_DIR=$(pwd)/build/lib
+    export S2N_TLS_INCLUDE_DIR=$(pwd)/api
+    echo "rust_test: Running Rust integration tests"
+    cargo test --manifest-path bindings/rust/standard/integration/Cargo.toml 
+)}
